@@ -14,6 +14,8 @@ import (
 	"strconv"
 	"strings"
 
+	"gopkg.in/yaml.v2"
+
 	"github.com/getkin/kin-openapi/openapi3"
 )
 
@@ -41,6 +43,8 @@ type ParseError struct {
 	path []interface{}
 }
 
+var _ interface{ Unwrap() error } = ParseError{}
+
 func (e *ParseError) Error() string {
 	var msg []string
 	if p := e.Path(); len(p) > 0 {
@@ -80,6 +84,10 @@ func (e *ParseError) RootCause() error {
 	return e.Cause
 }
 
+func (e ParseError) Unwrap() error {
+	return e.Cause
+}
+
 // Path returns a path to the root cause.
 func (e *ParseError) Path() []interface{} {
 	var path []interface{}
@@ -832,6 +840,8 @@ func decodeBody(body io.Reader, header http.Header, schema *openapi3.SchemaRef,
 func init() {
 	RegisterBodyDecoder("text/plain", plainBodyDecoder)
 	RegisterBodyDecoder("application/json", jsonBodyDecoder)
+	RegisterBodyDecoder("application/x-yaml", yamlBodyDecoder)
+	RegisterBodyDecoder("application/yaml", yamlBodyDecoder)
 	RegisterBodyDecoder("application/problem+json", jsonBodyDecoder)
 	RegisterBodyDecoder("application/x-www-form-urlencoded", urlencodedBodyDecoder)
 	RegisterBodyDecoder("multipart/form-data", multipartBodyDecoder)
@@ -854,6 +864,14 @@ func jsonBodyDecoder(body io.Reader, header http.Header, schema *openapi3.Schema
 	return value, nil
 }
 
+func yamlBodyDecoder(body io.Reader, header http.Header, schema *openapi3.SchemaRef, encFn EncodingFn) (interface{}, error) {
+	var value interface{}
+	if err := yaml.NewDecoder(body).Decode(&value); err != nil {
+		return nil, &ParseError{Kind: KindInvalidFormat, Cause: err}
+	}
+	return value, nil
+}
+
 func urlencodedBodyDecoder(body io.Reader, header http.Header, schema *openapi3.SchemaRef, encFn EncodingFn) (interface{}, error) {
 	// Validate schema of request body.
 	// By the OpenAPI 3 specification request body's schema must have type "object".
